@@ -3,13 +3,19 @@ use Mojo::Base 'Mojolicious::Plugin';
 use Time::HiRes qw/gettimeofday tv_interval/;
 use Net::Prometheus;
 
-our $VERSION = '0.9.4';
+our $VERSION = '1.0';
 
 has prometheus => sub { state $prom = Net::Prometheus->new };
 has route => sub {undef};
 has http_request_duration_seconds => sub {
     undef;
 };
+has http_request_size_bytes => sub {
+    undef;
+};
+has http_response_size_bytes => sub {
+    undef;
+};
 has http_requests_total => sub {
     undef;
 };
@@ -22,11 +28,33 @@ sub register {
             namespace => $config->{namespace} // undef,
             subsystem => $config->{subsystem} // undef,
             name      => "http_request_duration_seconds",
-            help      => "Summary request processing time",
+            help      => "Histogram with request processing time",
             labels    => [qw/method/],
         )
     );
 
+    $self->http_request_size_bytes(
+        $self->prometheus->new_histogram(
+            namespace => $config->{namespace} // undef,
+            subsystem => $config->{subsystem} // undef,
+            name      => "http_request_size_bytes",
+            help      => "Histogram containing request sizes",
+            labels    => [qw/method/],
+            buckets   => $config->{request_buckets} // [(1, 50, 100, 1_000, 10_000, 50_000, 100_000, 500_000, 1_000_000)],
+        )
+    );
+
+    $self->http_response_size_bytes(
+        $self->prometheus->new_histogram(
+            namespace => $config->{namespace} // undef,
+            subsystem => $config->{subsystem} // undef,
+            name      => "http_response_size_bytes",
+            help      => "Histogram containing response sizes",
+            labels    => [qw/method code/],
+            buckets   => $config->{response_buckets} // [(5, 50, 100, 1_000, 10_000, 50_000, 100_000, 500_000, 1_000_000)],
+        )
+    );
+
     $self->http_requests_total(
         $self->prometheus->new_counter(
             namespace => $config->{namespace} // undef,
@@ -50,6 +78,7 @@ sub register {
         before_dispatch => sub {
             my ($c) = @_;
             $c->stash( 'prometheus.start_time' => [gettimeofday] );
+            $self->http_request_size_bytes->observe( $c->req->method, $c->req->content->asset->size );
         }
     );
 
@@ -65,6 +94,7 @@ sub register {
         after_dispatch => sub {
             my ($c) = @_;
             $self->http_requests_total->inc( $c->req->method, $c->res->code );
+            $self->http_response_size_bytes->observe( $c->req->method, $c->res->code, $c->res->content->asset->size );
         }
     );
 
@@ -130,6 +160,18 @@ Override the L<Net::Prometheus> object. The default is a new singleton instance
 
 These will be prefixed to the metrics exported.
 
+=item * request_buckets
+
+Override buckets for request sizes histogram.
+Default:
+  (1, 50, 100, 1_000, 10_000, 50_000, 100_000, 500_000, 1_000_000)
+
+=item * response_buckets
+
+Override buckets for response sizes histogram.
+Default:
+  (5, 50, 100, 1_000, 10_000, 50_000, 100_000, 500_000, 1_000_000)
+
 =back
 
 =head1 METRICS
@@ -139,9 +181,13 @@ this plugin will also export
 
 =over 2
 
-=item C<http_requests_total>, counter partitioned over HTTP method and HTTP response code
+=item * C<http_requests_total>, request counter partitioned over HTTP method and HTTP response code
 
-=item C<http_request_duration_seconds>, histogram partitoned over HTTP method
+=item * C<http_request_duration_seconds>, request duration histogram partitoned over HTTP method
+
+=item * C<http_request_size_bytes>, request size histogram partitoned over HTTP method
+
+=item * C<http_response_size_bytes>, response size histogram partitoned over HTTP method
 
 =back
 
